@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2009, 2020 IBM Corp.
+ * Copyright (c) 2009, 2022 IBM Corp, Ian Craggs
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v2.0
@@ -1020,7 +1020,14 @@ int test4_run(int qos, int start_mqtt_version, int restore_mqtt_version)
 		}
 	}
 
-	MQTTClient_yield();  /* allow any unfinished protocol exchanges to finish */
+	/* call yield a few times until unfinished protocol exchanges are finished */
+	count = 0;
+	do
+	{
+		MQTTClient_yield();
+		rc = MQTTClient_getPendingDeliveryTokens(c, &tokens);
+		assert("getPendingDeliveryTokens rc == 0", rc == MQTTCLIENT_SUCCESS, "rc was %d", rc);
+	} while (tokens != NULL && ++count < 10);
 
 	rc = MQTTClient_getPendingDeliveryTokens(c, &tokens);
 	assert("getPendingDeliveryTokens rc == 0", rc == MQTTCLIENT_SUCCESS, "rc was %d", rc);
