@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2012, 2020 IBM Corp. and others
+ * Copyright (c) 2012, 2022 IBM Corp., Ian Craggs and others
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v2.0
@@ -88,12 +88,12 @@ struct Options
 	int websockets;
 } options =
 {
-	"ssl://m2m.eclipse.org:18883",
-	"ssl://m2m.eclipse.org:18884",
-	"ssl://m2m.eclipse.org:18887",
-	"ssl://m2m.eclipse.org:18885",
-	"ssl://m2m.eclipse.org:18886",
-	"ssl://m2m.eclipse.org:18888",
+	"ssl://localhost:18883",
+	"mqtts://localhost:18884",
+	"ssl://localhost:18887",
+	"mqtts://localhost:18885",
+	"ssl://localhost:18886",
+	"mqtts://localhost:18888",
 	NULL,
 	0,
 	"../../../test/ssl/client.pem",
