@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2012, 2020 IBM Corp. and others
+ * Copyright (c) 2012, 2022 IBM Corp., Ian Craggs
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v2.0
@@ -56,8 +56,8 @@ struct Options
 	int test_no;
 } options =
 {
-	"mqtt.eclipse.org:1883",
 	"localhost:1883",
+	"localhost:1884",
 	0,
 	0,
 };
